stream library

Classes

Browser
The browser.
DefaultRouter
The default implementation of Router.
ErrorDetail
The error detailed information.
HttpChannel
A HTTP channel. A channel represents the Internet address and port that StreamServer is bound to.
HttpConnect
A HTTP request connection.
HttpConnectWrapper
The HTTP connection wrapper. It simplifies the overriding of a connection.
Router
Router for mapping URI to renderers.
Rsp
RSP utilities.
StreamServer
Stream server.

Typedefs

PathPreprocessor = String Function(String path)
Used with StreamServer.pathPreprocessor for preprocessing the request's path.
RequestFilter = Future Function(HttpConnect connect, Future chain(HttpConnect conn))
The request filter. It is used with the filterMapping parameter of StreamServer.
RequestHandler = Future Function(HttpConnect conn)
The request handler.

Exceptions / Errors

Http204
HTTP 204 No Content. Throw to short-circuit a handler with an empty 204 response.
Http304
HTTP 304 Not Modified. Throw from a conditional-GET handler to short-circuit with an empty 304 response.
Http400
HTTP 400 exception.
Http401
HTTP 401 exception.
Http402
HTTP 402 exception.
Http403
HTTP 403 exception.
Http404
HTTP 404 exception.
Http405
HTTP 405 exception.
Http406
HTTP 406 exception.
Http408
HTTP 408 exception.
Http409
HTTP 409 exception.
Http410
HTTP 410 exception.
Http411
HTTP 411 exception.
Http412
HTTP 412 exception.
Http413
HTTP 413 exception.
Http414
HTTP 414 exception.
Http415
HTTP 415 exception.
Http417
HTTP 417 exception.
Http418
HTTP 418 exception.
Http421
HTTP 421 exception.
Http422
HTTP 422 Unprocessable Entity. Use for requests whose payload parses but fails semantic validation.
Http425
HTTP 425 exception.
Http426
HTTP 426 exception.
Http429
HTTP 429 exception.
Http431
HTTP 431 exception.
Http451
HTTP 451 exception.
Http500
HTTP 500 exception.
Http501
HTTP 501 exception.
Http502
HTTP 502 Bad Gateway. Use when an upstream/proxied request fails.
Http503
HTTP 503 exception.
Http504
HTTP 504 exception.
Http511
HTTP 511 exception.
HttpStatusException
A HTTP status exception.
ServerError
A generic server error.