Asset class abstract

An asset (aka., resource), such as a file or a BLOB object in database.

Note: It is used by loadAsset and AssetLoader (and derives). ResourceLoader doesn't depend on it.

Implementers

Constructors

Asset()

Properties

hashCode → int
The hash code for this object.
no setterinherited
path → String
The path uniquely identifies this asset. It must be unique in the scope of AssetLoader.
no setter
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited

Methods

lastModified() → Future<DateTime>
When this asset was last modified. *
length() → Future<int>
The length of this asset. *
noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
openRead([int? start, int? end]) → Stream<List<int>>
Create a new independent Stream for the contents of this asset. *
readAsBytes() → Future<List<int>>
Read the entire asset's contents as a list of bytes.
toString() → String
A string representation of this object.
inherited

Operators

operator ==(Object other) → bool
The equality operator.
inherited