FileAsset class

A file-based asset.

Implemented types

Constructors

FileAsset(File file)

Properties

file → File
final
hashCode → int
The hash code for this object.
no setteroverride
path → String
The path uniquely identifies this asset. It must be unique in the scope of AssetLoader.
no setteroverride
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited

Methods

lastModified() → Future<DateTime>
When this asset was last modified. *
override
length() → Future<int>
The length of this asset. *
override
noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
openRead([int? start, int? end]) → Stream<List<int>>
Create a new independent Stream for the contents of this asset. *
override
readAsBytes() → Future<List<int>>
Read the entire asset's contents as a list of bytes.
override
toString() → String
A string representation of this object.
override

Operators

operator ==(Object o) → bool
The equality operator.
override