Router class abstract

Router for mapping URI to renderers.

Implementers

Constructors

Router()

Properties

hashCode → int
The hash code for this object.
no setterinherited
protectRSP → bool
Whether RSP source files (.rsp and .rsp.*) are protected from being served to the client. The file loader honors this too as a backstop.
no setter
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited

Methods

filter(Pattern uri, RequestFilter filter, {bool preceding = false}) → void
Maps the given URI to the given filter.
getErrorHandler(Object? error) → Object?
Returns the error handler (RequestHandler) or a URI (String) based on the error thrown by a request handler.
getFilterAt(int iFilter) → RequestFilter
Returns the filter at the given index.
getFilterIndex(HttpConnect connect, String uri, int iFilter) → int?
Returns the index of the next matched request filter for the given URI and starting at the given index.
getHandler(HttpConnect connect, String uri) → Object?
Retrieves the first matched request handler (RequestHandler) or forwarded URI (String) for the given URI.
map(Pattern uri, Object? handler, {bool preceding = false}) → void
Maps the given URI to the given handler.
noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() → String
A string representation of this object.
inherited

Operators

operator ==(Object other) → bool
The equality operator.
inherited