filter abstract method
Maps the given URI to the given filter.
The interpretation of uri is really up to the implementation of Router.
filter- if filter is null, it means removal.preceding- whether to make the mapping preceding any previous mappings. In other words, if true, this mapping will be interpreted first.
Implementation
void filter(String uri, RequestFilter filter, {bool preceding = false});