map abstract method
Maps the given URI to the given handler.
The interpretation of uri and handler is really up to the
implementation of Router.
handler- if handler is null, it means removal.preceding- whether to make the mapping preceding any previous mappings. In other words, if true, this mapping will be interpreted first.
Implementation
void map(String uri, handler, {bool preceding = false});